#include "buffer.h"

Buffer creeBuffer(int w, int h) {
    Buffer buf;
    buf.z = new float[w*h];
    buf.im = Image<Color>(w,h);
    reinit(buf);
    return buf;
}

void reinit(Buffer& buf) {
    int w = buf.im.width();
    int h = buf.im.height();
    for(int j=0; j<h; j++)
        for(int i=0; i<w; i++) {
            buf.z[j*w+i] = -1e10f;
            buf.im(i,j) = WHITE;
        }
}

void affiche(Buffer buf) {
    display(buf.im);
}

void rectangle(Point pt[3], int& xmin, int& xmax, int& ymin, int& ymax) {
    float x=pt[0].x, X=x;
    float y=pt[0].y, Y=y;
    for(int i=1; i<3; i++) {
        if(x > pt[i].x)
            x = pt[i].x;
        else if(X < pt[i].x)
            X = pt[i].x;
        if(y > pt[i].y)
            y = pt[i].y;
        else if(Y < pt[i].y)
            Y = pt[i].y;
    }
    xmin = (int)x;
    xmax = (int)X+1;
    ymin = (int)y;
    ymax = (int)Y+1;
}

Point barycentre(Point a, Point b, Point c, float x, float y) {
    Point bary={-1.0f,-1.0f,-1.0f};
    float det=(b.x-a.x)*(c.y-a.y)-(b.y-a.y)*(c.x-a.x);
    if(-1.0f < det && det < 1.0f)
        return bary;
    bary.x = ((b.x-x)*(c.y-y)-(b.y-y)*(c.x-x))/det;
    bary.y = ((c.x-x)*(a.y-y)-(c.y-y)*(a.x-x))/det;
    bary.z = ((a.x-x)*(b.y-y)-(a.y-y)*(b.x-x))/det;
    return bary;
}

void projette(TriangleFace tr, Point* sommets, Buffer& buf) {
    int xmin, xmax, ymin, ymax;
    Point pt[] = {sommets[tr.index[0]],
                  sommets[tr.index[1]],
                  sommets[tr.index[2]]};
    rectangle(pt, xmin, xmax, ymin, ymax);
    for(int y=ymin; y<=ymax; y++) {
        if(y<0 || y>=buf.im.height())
            continue;
        for(int x=xmin; x<=xmax; x++) {
            if(x<0 || x>=buf.im.width())
                continue;
            Point bary = barycentre(pt[0],pt[1],pt[2], (float)x, (float)y);
            if(bary.x<0 || bary.y<0 || bary.z<0)
                continue;
            float z = bary.x*pt[0].z + bary.y*pt[1].z + bary.z*pt[2].z;
            if(buf.z[y*buf.im.width()+x]<z) {
                buf.z[y*buf.im.width()+x]=z;
                buf.im(x,y) = tr.c;
            }
        }
    }
}

void projette(Polyedre p, Buffer& buf) {
    for(int i=0; i<p.nTriangles; i++)
        projette(p.triangles[i], p.sommets, buf);
}
