#include "poly.h"
#include <cmath>

Point operator+(Point a, Point b) {
    Point p={a.x+b.x,a.y+b.y,a.z+b.z};
    return p;
}

Point operator-(Point a, Point b) {
    Point p={a.x-b.x,a.y-b.y,a.z-b.z};
    return p;
}

Point operator*(Point a, float f) {
    Point p={f*a.x,f*a.y,f*a.z};
    return p;
}

void set(TriangleFace& t, int i0, int i1, int i2, Color c) {
    t.index[0]=i0;
    t.index[1]=i1;
    t.index[2]=i2;
    t.c = c;
}

Color randCol() {
    return Color(rand()%256, rand()%256, rand()%256);
}

Point randomVect() {
    float alpha = rand()/(float)RAND_MAX*2*M_PI;
    float beta = rand()/(float)RAND_MAX*M_PI;
    Point v = {cos(alpha)*cos(beta),sin(alpha)*cos(beta),sin(beta)};
    return v;
}

void translate(Polyedre& p) {
    for(int i=0; i<p.nSommets; i++)
        p.sommets[i] = p.sommets[i]+p.centre;
}

void translate_inv(Polyedre& p) {
    for(int i=0; i<p.nSommets; i++)
        p.sommets[i] = p.sommets[i]-p.centre;
}

Polyedre tetraedre(int w, int h, int rmax) {
    Polyedre p;
    p.nSommets = 4;
    p.nTriangles = 4;
    p.sommets = new Point[p.nSommets];
    p.triangles = new TriangleFace[p.nTriangles];

    Point a={0,0,1.0f};
    Point b={1.0f,0.0f,0.0f};
    Point c={-1.0f/2,-sqrt(3.0f)/2, 0.0f};
    Point d={-1.0f/2,+sqrt(3.0f)/2, 0.0f};
    int r = rand()%rmax;
    p.sommets[0]=a*(float)r;
    p.sommets[1]=b*(float)r;
    p.sommets[2]=c*(float)r;
    p.sommets[3]=d*(float)r;

    set(p.triangles[0], 0,1,2, randCol());
    set(p.triangles[1], 0,1,3, randCol());
    set(p.triangles[2], 0,2,3, randCol());
    set(p.triangles[3], 1,2,3, randCol());

    p.centre.x = rand()/(float)RAND_MAX*w;
    p.centre.y = rand()/(float)RAND_MAX*h;
    p.centre.z = 0;

    translate(p);
    tourne(p, randomVect());
    return p;
}

Polyedre octaedre(int w, int h, int rmax) {
    Polyedre p;
    p.nSommets = 6;
    p.nTriangles = 8;
    p.sommets = new Point[p.nSommets];
    p.triangles = new TriangleFace[p.nTriangles];

    Point a={0,0,1};
    Point b={1,0,0};
    Point c={0,1,0};
    Point d={-1,0,0};
    Point e={0,-1,0};
    Point f={0,0,-1};
    int r = rand()%rmax;
    p.sommets[0]=a*(float)r;
    p.sommets[1]=b*(float)r;
    p.sommets[2]=c*(float)r;
    p.sommets[3]=d*(float)r;
    p.sommets[4]=e*(float)r;
    p.sommets[5]=f*(float)r;

    set(p.triangles[0], 0,1,2, randCol());
    set(p.triangles[1], 0,2,3, randCol());
    set(p.triangles[2], 0,3,4, randCol());
    set(p.triangles[3], 0,4,1, randCol());
    set(p.triangles[4], 5,1,2, randCol());
    set(p.triangles[5], 5,2,3, randCol());
    set(p.triangles[6], 5,3,4, randCol());
    set(p.triangles[7], 5,4,1, randCol());

    p.centre.x=rand()/(float)RAND_MAX*w;
    p.centre.y=rand()/(float)RAND_MAX*h;
    p.centre.z = 0;

    translate(p);
    tourne(p, randomVect());
    return p;
}

void tourne(Polyedre& p, Point vec) {
    float alpha = sqrt(vec.x*vec.x+vec.y*vec.y+vec.z*vec.z);
    if(alpha < 1.0e-2f) // Trop petit: mieux vaut ignorer que diviser par alpha
        return;
    vec = vec*(1/alpha);
    translate_inv(p);
    float c=cos(alpha), s=sin(alpha);
    for(int i=0; i<p.nSommets; i++) {
        Point pt=p.sommets[i];
        p.sommets[i].x = c*pt.x -s*vec.z*pt.y +s*vec.y*pt.z;
        p.sommets[i].y = c*pt.y +s*vec.z*pt.x -s*vec.x*pt.z;
        p.sommets[i].z = c*pt.z -s*vec.y*pt.x +s*vec.x*pt.y;
        float scal = vec.x*pt.x+vec.y*pt.y+vec.z*pt.z;
        p.sommets[i] = p.sommets[i] + vec*((1-c)*scal);
    }
    translate(p);
}

void detruit(Polyedre p) {
    delete [] p.sommets;
    delete [] p.triangles;
}
